/* 
   ipmi-openipmi-api-udm.c: IPMI UDM - OpenIPMI Api

   Copyright (C) 2003, 2004, 2005 FreeIPMI Core Team

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#ifdef STDC_HEADERS
#include <string.h>
#endif /* STDC_HEADERS */
#include <errno.h>

#include "freeipmi/udm/ipmi-openipmi-api-udm.h"
#include "freeipmi/ipmi-openipmi-api.h"

#include "ipmi-udm-device.h"

#include "err-wrappers.h"
#include "fiid-wrappers.h"
#include "freeipmi-portability.h"

fiid_template_t tmpl_openipmi_raw =
  {
    {8,    "cmd",      FIID_FIELD_REQUIRED | FIID_FIELD_LENGTH_FIXED},
    {8192, "raw_data", FIID_FIELD_OPTIONAL | FIID_FIELD_LENGTH_VARIABLE},
    {0, "", 0}
  };

int8_t 
ipmi_openipmi_cmd_udm (ipmi_device_t dev,
                       fiid_obj_t obj_cmd_rq,
                       fiid_obj_t obj_cmd_rs)
{
  ERR_EINVAL (dev
	      && dev->magic == IPMI_UDM_DEVICE_MAGIC
	      && fiid_obj_valid(obj_cmd_rq)
	      && fiid_obj_valid(obj_cmd_rs));

  FIID_OBJ_PACKET_VALID(obj_cmd_rq);

  ERR (!(ipmi_openipmi_cmd (dev->io.inband.openipmi_ctx,
                            dev->lun,
                            dev->net_fn,
                            obj_cmd_rq,
                            obj_cmd_rs) < 0));

  return (0);
}

int32_t
ipmi_openipmi_cmd_raw_udm (ipmi_device_t dev,
                           uint8_t *buf_rq,
                           size_t buf_rq_len,
                           uint8_t *buf_rs,
                           size_t buf_rs_len)
{
  fiid_obj_t obj_cmd_rq = NULL;
  fiid_obj_t obj_cmd_rs = NULL;
  int32_t len;
  int32_t rv = -1;

  ERR_EINVAL (dev 
	      && dev->magic == IPMI_UDM_DEVICE_MAGIC
	      && buf_rq 
	      && buf_rq_len > 0
	      && buf_rs 
	      && buf_rs_len > 0);

  FIID_OBJ_CREATE_CLEANUP(obj_cmd_rq, tmpl_openipmi_raw);
  FIID_OBJ_CREATE_CLEANUP(obj_cmd_rs, tmpl_openipmi_raw);

  FIID_OBJ_SET_ALL_CLEANUP(obj_cmd_rq, buf_rq, buf_rq_len);

  ERR_CLEANUP (!(ipmi_openipmi_cmd (dev->io.inband.openipmi_ctx,
                                    dev->lun,
                                    dev->net_fn,
                                    obj_cmd_rq,
                                    obj_cmd_rs) < 0));

  FIID_OBJ_GET_ALL_LEN_CLEANUP(len, obj_cmd_rs, buf_rs, buf_rs_len);

  rv = len;
 cleanup:
  if (obj_cmd_rq)
    fiid_obj_destroy(obj_cmd_rq);
  if (obj_cmd_rs)
    fiid_obj_destroy(obj_cmd_rs);
  return (rv);
}
